import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "DELETE",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	const user: UserSession | null = request.session;
	if (!user) {
		return jsonResponse(401, {
			message: "Unauthorized",
			error: "You must be logged in to delete a guild",
		});
	}

	const { id: guildId } = request.params;
	const { name: providedName } = request.query;

	if (!guildId || !providedName) {
		return jsonResponse(400, {
			message: "Missing parameters",
			error: "Both guild ID and name are required",
		});
	}

	const client: Client = cassandra.getClient();

	const result = await client.execute(
		"SELECT name, owner_id FROM guilds WHERE id = ?",
		[guildId],
		{ prepare: true },
	);

	if (result.rowLength === 0) {
		return jsonResponse(404, {
			message: "Not Found",
			error: "Guild does not exist",
		});
	}

	const row = result.first();

	if (row.owner_id !== user.id) {
		return jsonResponse(403, {
			message: "Forbidden",
			error: "You do not own this guild",
		});
	}

	if (row.name !== providedName) {
		return jsonResponse(409, {
			message: "Conflict",
			error: "Guild name does not match provided value",
		});
	}

	try {
		await client.execute("DELETE FROM guilds WHERE id = ?", [guildId], {
			prepare: true,
		});

		await client.execute(
			"DELETE FROM guilds_by_owner WHERE owner_id = ? AND name = ?",
			[user.id, row.name],
			{ prepare: true },
		);

		logger.custom(
			"[GUILD DELETE]",
			`(${guildId})`,
			`${row.name} - Owner: ${user.id}`,
			"31",
		);

		return jsonResponse(200, {
			message: "Guild deleted successfully",
			data: { id: guildId, name: row.name },
		});
	} catch (error) {
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to delete guild",
		});
	}
}

export { handler, routeDef };
