import { frontend } from "@config";
import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import { pika } from "@lib/pika";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
	needsBody: "json",
};

async function handler(
	request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	const user: UserSession | null = request.session;
	if (!user) {
		return jsonResponse(401, {
			message: "Unauthorized",
			error: "You must be logged in to create an invite",
		});
	}

	const { id: guildId } = request.params;
	if (!guildId) {
		return jsonResponse(400, {
			message: "Missing guild ID",
		});
	}

	const { max_uses, expires_at } = requestBody as {
		max_uses?: number;
		expires_at?: string | null;
	};

	const cassandraClient: Client = cassandra.getClient();

	// TODO: When permissions are implemented, check if the user has permission to create an invite
	const guild = await cassandraClient.execute(
		"SELECT owner_id FROM guilds WHERE id = ?",
		[guildId],
		{ prepare: true },
	);

	if (guild.rowLength === 0) {
		return jsonResponse(404, {
			message: "Guild not found",
		});
	}

	const guildData = guild.first();
	if (guildData.owner_id !== user.id) {
		return jsonResponse(403, {
			message: "Forbidden",
			error: "You are not the owner of this guild",
		});
	}

	const inviteCode = pika.gen("invite");
	const now = new Date();
	const expiresAt = expires_at ? new Date(expires_at) : null;

	try {
		await cassandraClient.execute(
			`INSERT INTO guild_invites (
				invite_code, guild_id, created_by, created_at,
				expires_at, max_uses, uses, is_revoked
			) VALUES (?, ?, ?, ?, ?, ?, ?, ?)`,
			[
				inviteCode,
				guildId,
				user.id,
				now,
				expiresAt,
				max_uses ?? null,
				0,
				false,
			],
			{ prepare: true },
		);

		logger.custom(
			"[GUILD INVITE]",
			`(${inviteCode})`,
			`Guild: ${guildId} by ${user.id}`,
			"35",
		);

		return jsonResponse(201, {
			message: "Invite created successfully",
			data: {
				url: `${frontend.origin}/invite/${inviteCode}`, // TODO: Update this to use the correct route ?
				invite: inviteCode,
				guild: guildId,
				expires_at: expiresAt,
				max_uses: max_uses ?? null,
			},
		});
	} catch (error) {
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to create invite",
		});
	}
}

export { handler, routeDef };
