import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "DELETE",
	accepts: "*/*",
	returns: "application/json;charset=utf-8",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	const user: UserSession | null = request.session;
	if (!user) {
		return jsonResponse(401, {
			message: "Unauthorized",
			error: "You must be logged in to leave a guild",
		});
	}

	const { id: guildId } = request.params;
	if (!guildId) {
		return jsonResponse(400, {
			message: "Missing guild ID",
		});
	}

	const client: Client = cassandra.getClient();

	const guild = await client.execute(
		"SELECT owner_id FROM guilds WHERE id = ?",
		[guildId],
		{ prepare: true },
	);

	if (guild.rowLength === 0) {
		return jsonResponse(404, {
			message: "Guild not found",
		});
	}

	const ownerId = guild.first().owner_id;
	if (ownerId === user.id) {
		return jsonResponse(403, {
			message: "You cannot leave a guild you own",
		});
	}

	const membership = await client.execute(
		"SELECT user_id FROM guild_members WHERE guild_id = ? AND user_id = ?",
		[guildId, user.id],
		{ prepare: true },
	);

	if (membership.rowLength === 0) {
		return jsonResponse(409, {
			message: "You are not a member of this guild",
		});
	}

	try {
		await client.execute(
			"DELETE FROM guild_members WHERE guild_id = ? AND user_id = ?",
			[guildId, user.id],
			{ prepare: true },
		);

		await client.execute(
			"DELETE FROM members_by_user WHERE user_id = ? AND guild_id = ?",
			[user.id, guildId],
			{ prepare: true },
		);

		logger.custom(
			"[GUILD LEAVE]",
			`(${guildId})`,
			`User ${user.id} left`,
			"33",
		);

		return jsonResponse(200, {
			message: "Left guild successfully",
			data: {
				guild_id: guildId,
			},
		});
	} catch (error) {
		logger.error(error as Error);
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to leave guild",
		});
	}
}

export { handler, routeDef };
