import { isValidGuildName } from "@/lib/validators/guild";
import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import { pika } from "@lib/pika";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
	needsBody: "json",
};

async function handler(
	request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	const { name } = requestBody as { name: string };

	const user: UserSession | null = request.session;
	if (!user) {
		return jsonResponse(401, {
			message: "Unauthorized",
			error: "You must be logged in to create a guild",
		});
	}

	const errors: string[] = [];
	const { valid, name: validName, error } = isValidGuildName(name.trim());

	if (!valid) {
		errors.push(error || "Invalid guild name");
	}

	if (errors.length > 0) {
		return jsonResponse(400, {
			message: "Validation failed",
			error: errors.join("; "),
		});
	}

	const cassandraClient: Client = cassandra.getClient();

	const existing = await cassandraClient.execute(
		"SELECT guild_id FROM guilds_by_owner WHERE owner_id = ? AND name = ?",
		[user.id, validName],
		{ prepare: true },
	);

	if (existing.rowLength > 0) {
		return jsonResponse(409, {
			message: "Conflict",
			error: "You already have a guild with this name",
		});
	}

	const guildId = pika.gen("guild");
	const now = new Date();

	try {
		await cassandraClient.execute(
			`INSERT INTO guilds (
				id, name, owner_id,
				created_at, updated_at
			) VALUES (?, ?, ?, ?, ?)`,
			[guildId, validName, user.id, now, now],
			{ prepare: true },
		);

		await cassandraClient.execute(
			`INSERT INTO guilds_by_owner (
				owner_id, name, guild_id
			) VALUES (?, ?, ?)`,
			[user.id, validName, guildId],
			{ prepare: true },
		);

		logger.custom(
			"[GUILD CREATE]",
			`(${guildId})`,
			`${validName} - Owner: ${user.id}`,
			"36",
		);

		return jsonResponse(201, {
			message: "Guild created successfully",
			data: {
				id: guildId,
				name: validName,
				owner: user.id,
				created_at: now,
				updated_at: now,
			},
		});
	} catch (error) {
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to create guild",
		});
	}
}

export { handler, routeDef };
