import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "*/*",
	returns: "application/json;charset=utf-8",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	const user: UserSession | null = request.session;
	if (!user) {
		return jsonResponse(401, {
			message: "Unauthorized",
			error: "You must be logged in to join a guild",
		});
	}

	const { invite } = request.params;

	if (!invite) {
		return jsonResponse(400, {
			message: "Missing invite code",
			error: "Invite code is required",
		});
	}

	const client: Client = cassandra.getClient();

	const inviteResult = await client.execute(
		"SELECT * FROM guild_invites WHERE invite_code = ?",
		[invite],
		{ prepare: true },
	);

	if (inviteResult.rowLength === 0) {
		return jsonResponse(404, {
			message: "Invalid invite code",
		});
	}

	const inviteData = inviteResult.first();

	if (inviteData.is_revoked) {
		return jsonResponse(403, {
			message: "This invite has been revoked",
		});
	}

	if (inviteData.expires_at && inviteData.expires_at < new Date()) {
		return jsonResponse(403, {
			message: "This invite has expired",
		});
	}

	if (inviteData.max_uses !== null && inviteData.uses >= inviteData.max_uses) {
		return jsonResponse(403, {
			message: "This invite has reached its usage limit",
		});
	}

	const guildId = inviteData.guild_id;

	const memberResult = await client.execute(
		"SELECT is_banned FROM guild_members WHERE guild_id = ? AND user_id = ?",
		[guildId, user.id],
		{ prepare: true },
	);

	if (memberResult.rowLength > 0) {
		const member = memberResult.first();
		if (member.is_banned) {
			return jsonResponse(403, {
				message: "You are banned from this guild",
			});
		}
		return jsonResponse(409, {
			message: "You are already a member of this guild",
		});
	}

	const now = new Date();

	try {
		await client.execute(
			`INSERT INTO guild_members (
				guild_id, user_id, roles, joined_at, is_banned, invite_id
			) VALUES (?, ?, ?, ?, ?, ?)`,
			[guildId, user.id, [], now, false, invite],
			{ prepare: true },
		);

		await client.execute(
			`INSERT INTO members_by_user (
				user_id, guild_id, roles, joined_at, is_banned, invite_id
			) VALUES (?, ?, ?, ?, ?, ?)`,
			[user.id, guildId, [], now, false, invite],
			{ prepare: true },
		);

		await client.execute(
			"UPDATE guild_invites SET uses = ? WHERE invite_code = ?",
			[inviteData.uses + 1, invite],
			{ prepare: true },
		);

		logger.custom(
			"[GUILD JOIN VIA INVITE]",
			`(${guildId})`,
			`User ${user.id} joined with invite ${invite}`,
			"32",
		);

		return jsonResponse(200, {
			message: "Joined guild successfully",
			data: {
				guild_id: guildId,
			},
		});
	} catch (error) {
		logger.error(error as Error);
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to join guild",
		});
	}
}

export { handler, routeDef };
