import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";

function toSafeUser(user: User, sessionId?: string): UserSafe | UserPrivate {
	const base: UserSafe = {
		id: user.id,
		username: user.username,
		display_name: user.display_name,
		avatar_url: user.avatar_url,
		is_verified: user.is_verified,
		created_at: user.created_at,
		updated_at: user.updated_at,
	};

	return sessionId === user.id ? { ...base, email: user.email } : base;
}

const routeDef: RouteDef = {
	method: "GET",
	accepts: "*/*",
	returns: "application/json;charset=utf-8",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	const { id: userId } = request.params;

	if (!userId) {
		return jsonResponse(400, { error: "Missing user ID in request." });
	}

	try {
		const client = cassandra.getClient();
		const query = `
			SELECT id, username, display_name, email, avatar_url, is_verified, created_at, updated_at
			FROM users
			WHERE id = ?
		`;
		const result = await client.execute(query, [userId], { prepare: true });

		if (result.rowLength === 0) {
			return jsonResponse(404, {
				error: "User not found.",
				message: "No user exists with the given ID.",
			});
		}

		const row = result.first();
		const user: User = {
			id: row.id,
			username: row.username,
			display_name: row.display_name,
			email: row.email,
			password: "", // this is never actually used or returned
			avatar_url: row.avatar_url,
			is_verified: row.is_verified,
			created_at: row.created_at,
			updated_at: row.updated_at,
		};

		return jsonResponse(200, { user: toSafeUser(user, request.session?.id) });
	} catch (err) {
		console.error(err);
		return jsonResponse(500, { error: "Internal server error." });
	}
}

export { handler, routeDef };
