import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import { sessionManager } from "@lib/jwt";
import { isValidEmail, isValidPassword } from "@lib/validators";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
	needsBody: "json",
};

async function handler(
	request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	const { email, password } = requestBody as {
		email: string;
		password: string;
	};
	const { force } = request.query;

	if (request.session && force !== "true" && force !== "1") {
		return jsonResponse(400, {
			message: "Already logged in",
			error: "You are already logged in",
		});
	}

	const errors: string[] = [];

	if (!email || typeof email !== "string") {
		errors.push("email is required");
	} else {
		const result = isValidEmail(email);
		if (!result.valid) errors.push(`email: ${result.error}`);
	}

	if (!password || typeof password !== "string") {
		errors.push("password is required");
	} else {
		const result = isValidPassword(password);
		if (!result.valid) errors.push(`password: ${result.error}`);
	}

	if (errors.length > 0) {
		return jsonResponse(400, {
			message: "Validation failed",
			error: errors.join("; "),
		});
	}

	const client: Client = cassandra.getClient();
	const result = await client.execute(
		"SELECT id, username, email, password FROM users WHERE email = ?",
		[email],
		{ prepare: true },
	);

	if (result.rowLength === 0) {
		return jsonResponse(401, {
			message: "Invalid credentials",
			error: "Invalid email or password",
		});
	}

	const user = result.first();
	const isPasswordValid = await Bun.password.verify(password, user.password);

	if (!isPasswordValid) {
		return jsonResponse(401, {
			message: "Invalid credentials",
			error: "Invalid email or password",
		});
	}

	const cookie = await sessionManager.createSession(
		{
			id: user.id,
			username: user.username,
			email: user.email,
			is_verified: false,
		},
		request.headers.get("User-Agent") || "unknown",
	);

	return Response.json(
		{
			status: "OK",
			message: "Login successful",
			data: {
				id: user.id,
				username: user.username,
				email: user.email,
			},
		},
		{
			status: 200,
			headers: {
				"Content-Type": "application/json;charset=utf-8",
				"Set-Cookie": cookie,
			},
		},
	);
}

export { handler, routeDef };
