import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import { pika } from "@lib/pika";
import {
	isValidEmail,
	isValidPassword,
	isValidUsername,
} from "@lib/validators";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
	needsBody: "json",
};

async function handler(
	_request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	const { username, password, email } = requestBody as {
		username: string;
		password: string;
		email: string;
	};

	const fields = { username, password, email };

	const validators = {
		username: isValidUsername,
		password: isValidPassword,
		email: isValidEmail,
	};

	const errors: string[] = [];

	for (const [key, validate] of Object.entries(validators)) {
		const value = fields[key as keyof typeof fields];
		if (typeof value !== "string" || value.trim() === "") {
			errors.push(`${key} is required`);
			continue;
		}

		const result = validate(value);
		if (!result.valid) errors.push(`${key}: ${result.error}`);
	}

	if (errors.length > 0) {
		return jsonResponse(400, {
			message: "Validation failed",
			error: errors.join("; "),
		});
	}

	const cassandraClient: Client = cassandra.getClient();

	const usernameResult = await cassandraClient.execute(
		"SELECT id FROM users WHERE username = ?",
		[username],
		{
			prepare: true,
		},
	);

	const emailResult = await cassandraClient.execute(
		"SELECT id FROM users WHERE email = ?",
		[email],
		{
			prepare: true,
		},
	);

	if (usernameResult.rowLength > 0 || emailResult.rowLength > 0) {
		const errorMessages: string[] = [];

		if (usernameResult.rowLength > 0) {
			errorMessages.push("Username has already been taken");
		}
		if (emailResult.rowLength > 0) {
			errorMessages.push("Email has already been taken");
		}

		return jsonResponse(400, {
			message: "Validation failed",
			error: errorMessages.join("; "),
		});
	}

	const user: UserInsert = {
		id: pika.gen("user"),
		username,
		email,
		password: await Bun.password.hash(password),
		created_at: new Date(),
		updated_at: new Date(),
	};

	try {
		await cassandraClient.execute(
			"INSERT INTO users (id, username, email, password, created_at, updated_at) VALUES (?, ?, ?, ?, ?, ?)",
			[
				user.id,
				user.username,
				user.email,
				user.password,
				user.created_at,
				user.updated_at,
			],
			{ prepare: true },
		);

		logger.custom(
			"[REGISTER]",
			`(${user.id})`,
			`${user.username} - ${user.email}`,
			"34",
		);
	} catch (error) {
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to register user",
		});
	}

	return jsonResponse(201, {
		message: "User registered successfully",
		data: {
			id: user.id,
			username: user.username,
			email: user.email,
		},
	});
}

export { handler, routeDef };
